==> ecomero-businesscentral-1.2.30.zip: Model/Erp/BcCustomer.php <==
<?php

declare(strict_types=1);
/**
 *           a88888P8
 *          d8'
 * .d8888b. 88        .d8888b. 88d8b.d8b. .d8888b. .dd888b. .d8888b.
 * 88ooood8 88        88'  `88 88'`88'`88 88ooood8 88'    ` 88'  `88
 * 88.  ... Y8.       88.  .88 88  88  88 88.  ... 88       88.  .88
 * `8888P'   Y88888P8 `88888P' dP  dP  dP `8888P'  dP       `88888P'.
 *
 *           Copyright © eComero Management AB, All rights reserved.
 */

namespace Ecomero\BusinessCentral\Model\Erp;

use Ecomero\BusinessCentral\Helper\Settings;
use Ecomero\BusinessCentral\Model\BusinessCentralAdapter;
use Ecomero\BusinessCentral\Model\BusinessCentralAPI;
use Ecomero\ErpCore\Model\Capability;
use Ecomero\ErpCore\Model\Erp\ErpCustomerInterface;
use Ecomero\ErpCore\Model\Order\ResponseFactory as OrderResponseFactory;
use Magento\Framework\Stdlib\ArrayManager;

class BcCustomer extends BusinessCentralAdapter implements ErpCustomerInterface
{
    public function __construct(
        BusinessCentralAPI $api,
        Settings $settings,
        Capability $capability,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        OrderResponseFactory $orderResponseFactory,
        ArrayManager $arrayManager
    ) {
        parent::__construct(
            $api,
            $settings,
            $capability,
            $storeManager,
            $orderResponseFactory,
            $arrayManager
        );
    }

    public function getCompanyList(): array
    {
        $companies = [];

        $endpoint = $this->api->getEndPointForAPI('/api/v2.0').'customers';
        $rc = $this->api->sendRequest($endpoint);
        $customers = $this->arrayManager->get('value', $rc);

        $endpoint = $this->api->getEndPointForAPI('/api/v2.0')."contacts?\$filter=contactBusinessRelation eq 'Customer'";
        $rc = $this->api->sendRequest($endpoint);
        $contacts = $this->arrayManager->get('value', $rc);

        foreach ($customers as $companyRecord) {
            $type = $this->arrayManager->get('type', $companyRecord);

            if ('Company' !== $type) {
                continue;
            }

            $customer = [];

            // Create company
            $company['id'] = $this->arrayManager->get('number', $companyRecord);
            $company['number'] = $this->arrayManager->get('number', $companyRecord);
            $company['name'] = $this->arrayManager->get('displayName', $companyRecord);
            $company['alternativeName'] = $this->arrayManager->get('displayName', $companyRecord);
            $company['email'] = $this->arrayManager->get('email', $companyRecord);
            $company['web'] = $this->arrayManager->get('website', $companyRecord);
            $company['street'] = $company['deliveryStreet'] = $this->arrayManager->get('addressLine1', $companyRecord);
            $company['city'] = $company['deliveryCity'] = $this->arrayManager->get('city', $companyRecord);
            $company['countryId'] = $this->arrayManager->get('country', $companyRecord);
            $company['postalCode'] = $company['deliveryPostalCode'] = $this->arrayManager->get('postalCode', $companyRecord);
            $company['telephone'] = $this->arrayManager->get('phoneNumber', $companyRecord);
            $company['orgNo'] = $this->arrayManager->get('taxRegistrationNumber', $companyRecord);
            $company['vatNo'] = $this->arrayManager->get('taxRegistrationNumber', $companyRecord);
            $company['agreement'] = '';
            $company['priceList'] = $this->arrayManager->get('number', $companyRecord);
            $company['customers'] = [];

            $id = $this->arrayManager->get('id', $companyRecord);
            $endpoint = $this->api->getEndPointForAPI('/api/v2.0').'customers('.$id.')/contactsInformation';
            $rc = $this->api->sendRequest($endpoint);
            $customersContacts = $this->arrayManager->get('value', $rc);
            // Add contact persons to the company
            foreach ($customersContacts as $customersContact) {
                $type = $this->arrayManager->get('contactType', $customersContact);
                $contactNumber = $this->arrayManager->get('contactNumber', $customersContact);

                if ('Person' === $type) {
                    foreach ($contacts as $contact) {
                        if ($contactNumber !== $this->arrayManager->get('number', $contact)) {
                            continue;
                        }
                        $customer = [];

                        $names = $this->splitName($this->arrayManager->get('displayName', $contact));
                        $customer['id'] = $this->arrayManager->get('id', $contact);
                        $customer['number'] = $this->arrayManager->get('number', $contact);
                        $customer['firstName'] = $names['firstName'];
                        $customer['lastName'] = $names['lastName'];
                        $customer['street'] = $this->arrayManager->get('addressLine1', $contact);
                        $customer['city'] = $this->arrayManager->get('city', $contact);
                        $customer['postalCode'] = $this->arrayManager->get('postalCode', $contact);
                        $customer['email'] = $this->arrayManager->get('email', $contact);
                        $customer['companyId'] = $this->arrayManager->get('companyNumber', $contact);
                        $customer['telephoneSwitch'] = $this->arrayManager->get('phoneNumber', $contact);
                        $customer['telephoneDirect'] = $this->arrayManager->get('', $contact);
                        $customer['telephoneMobile'] = $this->arrayManager->get('mobilePhoneNumber', $contact);
                        $customer['countryId'] = $this->arrayManager->get('country', $contact);

                        $company['customers'][] = $customer;
                    }
                }
            }
            $companies[] = $company;
        }

        return $companies;
    }

    public function getCustomerCategories(): array
    {
        return [];
    }

    public function getCustomerDistricts(): array
    {
        return [];
    }

    public function getCustomerFromEmail(
        string $email
    ): array {
        $tmpResult = [];

        $rc = $this->api->callStandardApi('customers?$filter=email eq \''.$email.'\'');

        foreach ($rc as $item) {
            $newItem = [];
            $oldItem = $item;

            foreach ($oldItem as $key => $val) {
                if ('addressLine1' === $key) {
                    $newItem['street'] = $val;
                } elseif ('displayName' === $key) {
                    $newItem['name'] = $val;
                    $part = explode(' ', $val);
                    if (array_key_exists(0, $part)) {
                        $newItem['first_name'] = $part[0];
                    }
                    if (array_key_exists(1, $part)) {
                        $newItem['last_name'] = $part[1];
                    }
                } elseif ('phoneNumber' === $key) {
                    $newItem['phone'] = $val;
                } else {
                    $newItem[$key] = $val;
                }
            }

            $tmpResult[] = $newItem;
        }

        return $tmpResult;
    }

    public function createCustomer(
        ?string $company,
        ?string $firstName,
        ?string $lastName,
        ?string $street,
        ?string $city,
        ?string $postCode,
        ?string $email,
        ?string $phone,
        ?string $country,
        ?string $region
    ): array {
        $post_data = json_encode(['displayName' => $firstName.' '.$lastName,
            'phoneNumber' => $phone ?? '',
            'email' => $email,
            'address' => [
                'street' => $street,
                'city' => $city,
                'state' => $region,
                'countryLetterCode' => $country ?? '',
                'postalCode' => $postCode ?? '',
            ],  ]);

        return $this->api->callStandardApi('customers', $post_data);
    }

    private function splitName(string $name)
    {
        $name = trim($name);
        if (false === strpos($name, ' ')) {
            return ['firstName' => $name, 'lastName' => ''];
        }

        $parts = explode(' ', $name);
        $lastname = array_pop($parts);
        $firstname = implode(' ', $parts);

        return ['firstName' => $firstname, 'lastName' => $lastname];
    }
}

==> ecomero-businesscentral-1.2.62.zip: Model/Erp/BcCustomer.php <==
<?php

declare(strict_types=1);
/**
 *           a88888P8
 *          d8'
 * .d8888b. 88        .d8888b. 88d8b.d8b. .d8888b. .dd888b. .d8888b.
 * 88ooood8 88        88'  `88 88'`88'`88 88ooood8 88'    ` 88'  `88
 * 88.  ... Y8.       88.  .88 88  88  88 88.  ... 88       88.  .88
 * `8888P'   Y88888P8 `88888P' dP  dP  dP `8888P'  dP       `88888P'.
 *
 *           Copyright © eComero Management AB, All rights reserved.
 */

namespace Ecomero\BusinessCentral\Model\Erp;

use Ecomero\BusinessCentral\Helper\Settings;
use Ecomero\BusinessCentral\Model\BusinessCentralAdapter;
use Ecomero\BusinessCentral\Model\BusinessCentralAPI;
use Ecomero\ErpCore\Model\Capability;
use Ecomero\ErpCore\Model\Erp\ErpCustomerInterface;
use Ecomero\ErpCore\Model\Order\ResponseFactory as OrderResponseFactory;
use Magento\Framework\Stdlib\ArrayManager;
use Ecomero\ErpCore\Helper\ErpLogger;
class BcCustomer extends BusinessCentralAdapter implements ErpCustomerInterface
{
    protected $logger;

    public function __construct(
        BusinessCentralAPI $api,
        Settings $settings,
        Capability $capability,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        OrderResponseFactory $orderResponseFactory,
        ArrayManager $arrayManager,
        ErpLogger $logger
    ) {
        parent::__construct(
            $api,
            $settings,
            $capability,
            $storeManager,
            $orderResponseFactory,
            $arrayManager
        );
        $this->logger = $logger;
    }

    public function getCompanyList(): array
    {
        $companies = [];

        $this->logger->debug('...fetching companies');
        $endpoint = $this->api->getEndPointForAPI('/api/v2.0').'customers';
        $rc = $this->api->sendRequest($endpoint);
        $customers = $this->arrayManager->get('value', $rc);

        $this->logger->debug('...fetching company - contact relations');
        $endpoint = $this->api->getEndPointForAPI('/api/v2.0')."contacts?\$filter=contactBusinessRelation eq 'Customer'";
        $rc = $this->api->sendRequest($endpoint);
        $contacts = $this->arrayManager->get('value', $rc);

        $count = 0;
        $maxCount = count($customers);
        foreach ($customers as $companyRecord) {
            $count ++;

            $type = $this->arrayManager->get('type', $companyRecord);

            if ('Company' !== $type) {
                continue;
            }

            $customer = [];

            // Create company
            $company['id'] = $this->arrayManager->get('number', $companyRecord);
            $company['number'] = $this->arrayManager->get('number', $companyRecord);
            $company['name'] = $this->arrayManager->get('displayName', $companyRecord);
            $company['alternativeName'] = $this->arrayManager->get('displayName', $companyRecord);
            $company['email'] = $this->arrayManager->get('email', $companyRecord);
            $company['web'] = $this->arrayManager->get('website', $companyRecord);
            $company['street'] = $company['deliveryStreet'] = $this->arrayManager->get('addressLine1', $companyRecord);
            $company['city'] = $company['deliveryCity'] = $this->arrayManager->get('city', $companyRecord);
            $company['countryId'] = $this->arrayManager->get('country', $companyRecord);
            $company['postalCode'] = $company['deliveryPostalCode'] = $this->arrayManager->get('postalCode', $companyRecord);
            $company['telephone'] = $this->arrayManager->get('phoneNumber', $companyRecord);
            $company['orgNo'] = $this->arrayManager->get('taxRegistrationNumber', $companyRecord);
            $company['vatNo'] = $this->arrayManager->get('taxRegistrationNumber', $companyRecord);
            $company['agreement'] = '';
            $company['priceList'] = $this->arrayManager->get('number', $companyRecord);
            $company['customers'] = [];

            $this->logger->debug("...fetching {$count}/{$maxCount} contact information for {$company['name']}");
            $id = $this->arrayManager->get('id', $companyRecord);
            $endpoint = $this->api->getEndPointForAPI('/api/v2.0').'customers('.$id.')/contactsInformation';
            $rc = $this->api->sendRequest($endpoint);
            $customersContacts = $this->arrayManager->get('value', $rc);
            // Add contact persons to the company
            foreach ($customersContacts as $customersContact) {
                $type = $this->arrayManager->get('contactType', $customersContact);
                $contactNumber = $this->arrayManager->get('contactNumber', $customersContact);

                if ('Person' === $type) {
                    foreach ($contacts as $contact) {
                        if ($contactNumber !== $this->arrayManager->get('number', $contact)) {
                            continue;
                        }
                        $customer = [];

                        $names = $this->splitName($this->arrayManager->get('displayName', $contact));
                        $customer['id'] = $this->arrayManager->get('id', $contact);
                        $customer['number'] = $this->arrayManager->get('number', $contact);
                        $customer['firstName'] = $names['firstName'];
                        $customer['lastName'] = $names['lastName'];
                        $customer['street'] = $this->arrayManager->get('addressLine1', $contact);
                        $customer['city'] = $this->arrayManager->get('city', $contact);
                        $customer['postalCode'] = $this->arrayManager->get('postalCode', $contact);
                        $customer['email'] = $this->arrayManager->get('email', $contact);
                        $customer['companyId'] = $this->arrayManager->get('companyNumber', $contact);
                        $customer['telephoneSwitch'] = $this->arrayManager->get('phoneNumber', $contact);
                        $customer['telephoneDirect'] = $this->arrayManager->get('', $contact);
                        $customer['telephoneMobile'] = $this->arrayManager->get('mobilePhoneNumber', $contact);
                        $customer['countryId'] = $this->arrayManager->get('country', $contact);

                        $company['customers'][] = $customer;
                    }
                }
            }
            $companies[] = $company;
        }

        return $companies;
    }

    public function getCustomerCategories(): array
    {
        return [];
    }

    public function getCustomerDistricts(): array
    {
        return [];
    }

    public function getCustomerFromEmail(
        string $email
    ): array {
        $tmpResult = [];

        $rc = $this->api->callStandardApi('customers?$filter=email eq \''.$email.'\'');

        foreach ($rc as $item) {
            $newItem = [];
            $oldItem = $item;

            foreach ($oldItem as $key => $val) {
                if ('addressLine1' === $key) {
                    $newItem['street'] = $val;
                } elseif ('displayName' === $key) {
                    $newItem['name'] = $val;
                    $part = explode(' ', $val);
                    if (array_key_exists(0, $part)) {
                        $newItem['first_name'] = $part[0];
                    }
                    if (array_key_exists(1, $part)) {
                        $newItem['last_name'] = $part[1];
                    }
                } elseif ('phoneNumber' === $key) {
                    $newItem['phone'] = $val;
                } else {
                    $newItem[$key] = $val;
                }
            }

            $tmpResult[] = $newItem;
        }

        return $tmpResult;
    }

    public function createCustomer(
        ?string $company,
        ?string $firstName,
        ?string $lastName,
        ?string $street,
        ?string $city,
        ?string $postCode,
        ?string $email,
        ?string $phone,
        ?string $country,
        ?string $region
    ): array {
        $post_data = json_encode(['displayName' => $firstName.' '.$lastName,
            'phoneNumber' => $phone ?? '',
            'email' => $email,
            'address' => [
                'street' => $street,
                'city' => $city,
                'state' => $region,
                'countryLetterCode' => $country ?? '',
                'postalCode' => $postCode ?? '',
            ],  ]);

        return $this->api->callStandardApi('customers', $post_data);
    }

    private function splitName(string $name)
    {
        $name = trim($name);
        if (false === strpos($name, ' ')) {
            return ['firstName' => $name, 'lastName' => ''];
        }

        $parts = explode(' ', $name);
        $lastname = array_pop($parts);
        $firstname = implode(' ', $parts);

        return ['firstName' => $firstname, 'lastName' => $lastname];
    }
}
